#!/usr/bin/php
<?php
if (php_sapi_name() !== "cli")
  exit("Must be run from CLI");

// Initialization

define("FIND", "/usr/bin/find");
define("GZIP", "/usr/bin/gzip");

define("ROOT", dirname($_SERVER['SCRIPT_FILENAME']));
define("SITE", $argv[1]);
if (isset($argv[2]))
  define("DESTINATION", $argv[2]);
else
  define("DESTINATION", "local");

if (file_exists(SITE . "/config.ini"))
  $config = parse_ini_file(SITE . "/config.ini");

if (!isset($config['trueBlack']))
  $config['trueBlack'] = false;

if (!isset($config['siteTitle']))
  $config['siteTitle'] = "Site";

if (!isset($config['css']))
  $config['css'] = true;

if (!isset($config['header']))
  $config['header'] = true;

if (!isset($config['centerIndex']))
  $config['centerIndex'] = false;


// Less > CSS

require ROOT . "/bibli/less.php/lib/Less/Autoloader.php";
Less_Autoloader::register();

$colorScheme = array(
  "darkColor" => "#2a2a2a",
  "darkerColor" => "#222222",
  "lightColor" => "white",
  "lightlessColor" => "#eeeeee",
  "mainColor" => "red",
);

if ($config['trueBlack']) {
  $colorScheme['darkColor'] = "#000000";
  $colorScheme['darkerColor'] = "#000000";
}

$options = array('cache_dir' => SITE . '/css', 'compress' => true);

if (file_exists(SITE . "/style.less"))
  $lessFiles = array(ROOT . '/style.less' => '', SITE . '/style.less' => '');
else
  $lessFiles = array(ROOT . '/style.less' => '');

define("CSS_FILENAME", Less_Cache::Get($lessFiles, $options, $colorScheme));

// URLs don't end with .html on the Antopie website
function formerUrlLocale($page) {
  if (DESTINATION === "dns" OR DESTINATION === "onion") {
    echo $page;
  } else {
    echo $page . ".html";
  }
}

// Determine whether links need to use Onion or DNS
function clearnetOrOnion($clearnetUrl, $onionUrl) {
  if (DESTINATION === "onion") {
    return $onionUrl;
  } else {
    return $clearnetUrl;
  }
}

exec(FIND . " " . SITE . "/src -name '*.gmi' -o -name '*.md'", $pages);

foreach ($pages as $page) {

  $pathParts = pathinfo(str_replace("/src", "", $page));

  // Create parent directory if needed
  if (!file_exists($pathParts['dirname']))
    mkdir($pathParts['dirname'], 0755, true);

  // Execute PHP code
  ob_start();
  eval("?>" . file_get_contents($page));
  file_put_contents($pathParts['dirname'] . "/" . $pathParts['basename'], ob_get_contents());
  ob_end_clean();

  // Convert Gemtext to Markdown
  if ($pathParts['extension'] === "gmi") {
    $gmilines = explode("\n", file_get_contents($pathParts['dirname'] . "/" . $pathParts['basename']));

    if (substr($gmilines[0], 0, 2) === "# ")
      $title = substr($gmilines[0], 2);
    else
      $title = NULL;

    foreach ($gmilines as $key => $line) {
      if (substr($line, 0, 2) === "=>") {
        preg_match("/=> +(.[^ ]+)/", $line, $lnUrl);
        preg_match("/=> +.[^ ]+ +(.+)/", $line, $lnTitle);

        // Escape Markdown special characters
        $mdSpecial = array("[", "]", "(", ")");
        $htmlEntities = array("&#91;", "&#93;", "&#40;", "&#41;");
        $lnUrl[1] = str_replace($mdSpecial, $htmlEntities, $lnUrl[1]);

        $urlPathParts = pathinfo(parse_url($lnUrl[1], PHP_URL_PATH));

        // .gmi > .md for local links
        if (!str_contains($lnUrl[1], ":") AND $urlPathParts['extension'] === "gmi") // If it's a local link
          $lnUrl[1] = $urlPathParts['dirname'] . "/" . $urlPathParts['filename'] . ".md";

        if (isset($lnTitle[1])) {
          $lnTitle[1] = str_replace($mdSpecial, $htmlEntities, $lnTitle[1]);
          $gmilines[$key] = "[" . $lnTitle[1] . "](" . $lnUrl[1] . ")";
        } else {
          $gmilines[$key] = "[" . $lnUrl[1] . "](" . $lnUrl[1] . ")";
        }
      }
    }
    $code = "";
    foreach ($gmilines as $line) {
      $code = $code . "\n" . $line;
    }
    file_put_contents($pathParts['dirname'] . "/" . $pathParts['filename'] . ".md", $code);
  }


  // Compile Markdown to HTML with Parsedown
  require_once ROOT . "/bibli/parsedown/Parsedown.php";
  require_once ROOT . "/bibli/parsedown-extra/ParsedownExtra.php";
  $Parsedown = new ParsedownExtra;
  $Parsedown = $Parsedown->setUrlsLinked(false);
  $Parsedown = $Parsedown->setMarkupEscaped(false);
  $Parsedown = $Parsedown->setBreaksEnabled(true);
  $pageContent = $Parsedown->text(file_get_contents($pathParts['dirname'] . "/" . $pathParts['filename'] . ".md"));


  // .md > .html for local links
  $pageContent = preg_replace('#<a href="(?!.*:)(.*)\.md">#', '<a href="$1.html">', $pageContent);


  // Add header and footer to HTML

  ob_start();

  ?>
  <!DOCTYPE html>
  <html lang="fr">
    <head>
      <meta charset="UTF-8">
      <title><?php
      if (isset($title) AND !is_null($title))
        echo $title . " · " . $config['siteTitle'];
      else
        echo $config['siteTitle'];
      ?></title>
      <meta name="viewport" content="width=device-width, initial-scale=1">
      <?php if ($config['css']) { ?>
        <link type="text/css" rel="stylesheet" media="screen" href="/css/<?= CSS_FILENAME ?>">
      <?php } ?>
      <?php
      if (file_exists(SITE . "/head.inc.html"))
        echo file_get_contents(SITE . "/head.inc.html");
      ?>
    </head>

    <body>
      <?php
      if ($config['header']) {
      ?>
      <header>
        <a id="lienHeader" href="/">
          <div class="logo">
          <?php
          if (file_exists(SITE . "/img/logo.webp"))
            echo '<img src="img/logo.webp" ' . getimagesize(SITE . "/img/logo.webp")[3] . ' alt="' . $config['siteTitle'] . '" />';
          else
            echo $config['siteTitle'];
          ?>
          </div>
        </a>
      </header>
      <?php
      }
      ?>

      <main>

  <?php
  if ($config['centerIndex'] AND $pathParts['filename'] === "index") {
    echo "<div class='centre'>";
  } else {
    echo "<article>";
  }
  echo $pageContent;
  if ($config['centerIndex'] AND $pathParts['filename'] === "index") {
    echo "</div>";
  } else {
    echo "</article>";
  }
  echo "</main>";
  if (file_exists(SITE . "/end.inc.html"))
    require SITE . "/end.inc.html";
  echo "</body></html>";
  file_put_contents($pathParts['dirname'] . "/" . $pathParts['filename'] . ".html", ob_get_contents());
  ob_end_clean();

  // Gzip compression
  exec(GZIP . " --keep --fast --force " . $pathParts['dirname'] . "/" . $pathParts['filename'] . ".html");
}

exec(GZIP . " --keep --fast --force " . SITE . "/css/" . CSS_FILENAME);
